from fastapi import FastAPI
import uvicorn


class ToolRunner:
    def __init__(self, Tool, tool_info):
      # self.tool = Tool()
      self.app = FastAPI()
      self.tool_info = tool_info
      self.Tool = Tool
      # self.funcHash = {}
      # self.generateFuncHash()
      self.setupRoutes()
    
    def getToolMetadata(self):
      functions_list = []
      for action in self.Tool.__kl_function_metadata__:
        functions_list.append({
          'name': self.Tool.__kl_function_metadata__[action]['name'],
          'description': self.Tool.__kl_function_metadata__[action]['description'],
          'parameters': self.Tool.__kl_function_metadata__[action]['parameters'],
        })
      return {
          'functions': functions_list,
          'name': self.tool_info['name'],
          'description': self.tool_info['description'],
        }

    # def generateFuncHash(self):
    #   for param in dir(self.tool):
    #     if param.startswith('__'):
    #       continue
    #     if callable(getattr(self.tool, param)):
    #       fun = getattr(self.tool, param)
    #       self.funcHash[fun.__kl__name__] = fun

    def setupRoutes(self):
      @self.app.get('/functions')
      async def functions():
          functions = self.getToolMetadata()
          return {
              'functions': functions
            }

      @self.app.post('/trigger')
      async def trigger(data: dict):
          function_name = data['function_name']
          kwargs = data.get('arguments', {})
          params = data.get("tool_params", [])
          if not function_name:
            return {'error': 'function_name is required'}, 400
          t = self.Tool(**params)
          for m in dir(t):
            if m.startswith('__'):
              continue
            if callable(getattr(t, m)):
              fun = getattr(t, m)
              if fun.__kl__name__ == function_name:
                func = fun
                break
          if not func:
            return {'error': f'Function {function_name} not found'}, 404
          if not callable(func):
            return {'error': f'{function_name} is not callable'}, 400
          try:
            result = func(**kwargs)
          except Exception as e:
            return {'error': str(e)}, 500
          return {'result': result}